==> pkg_add_it-1.3.tar.gz: pkg_add_it-1.3/scan_dir.c <==
/*-
 * Copyright (c) 2010  Marin Atanasov Nikolov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>

#include <stdio.h>
#include <regex.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>

#include "main.h"
#include "pkg.h"
#include "scan_dir.h"

/* dir_exists
 *
 * checks for existence of a directory 
 */
int
dir_exists(const char *dirname)
{
   struct stat dir_stat;
   
   stat (dirname, &dir_stat);
   
   return S_ISDIR(dir_stat.st_mode);
}

/* scan_directory
 *
 * Recursively scans the specified directory for the package pattern.
 *
 * During local package search the pkg_fields[PKG_CAT] element is
 * used to store the full path to the package location.
 *
 * Returns the number of found packages.
 */
int
scan_directory(const char *pathname, const char *pattern, int options)
{
   PKG	  *entry;
   regex_t myregex;
   DIR	  *dir;
   char	   buf[MAXPATHLEN + 1]; 
   struct  dirent *dir_entry;
   struct  stat dir_stat;
   int     founddotdir, found_pkg, i;
   static unsigned count = 0;
   
   entry = (PKG *) malloc(sizeof(PKG));

   if (entry == NULL) {
      fprintf(stderr, "%s: Cannot allocate memory.\n", PROGNAME);
      ERRX;
   }

   entry->flags = 0;
   entry->pkg_num = 0;
   STAILQ_INIT(&entry->deps_head);

   for (i = 0; i < PKG_FIELDS; i++)
      entry->pkg_fields[i] = NULL;

   if (options & OPTION_REGEX)
      if (regcomp(&myregex, pattern, REG_ICASE | REG_NOSUB | \
	         ((options & OPTION_REGEX_EXTENDED) ? REG_EXTENDED : REG_BASIC)) != 0) {
	 fprintf(stderr, "%s: Cannot compile the given regex.\n", PROGNAME);
	 ERRX;
      }

   if (!(dir = opendir(pathname))) {
      WARNX;
      fprintf(stderr, "%s: Skipping directory %s\n", PROGNAME, pathname);
      return(0);
   }

   if (chdir(pathname) == -1) {
      WARNX;
      fprintf(stderr, "%s: Skipping directory %s\n", PROGNAME, pathname);
      return(0);
   }
   
   while ((dir_entry = readdir(dir))) {
      found_pkg = 0;

      if (stat(dir_entry->d_name, &dir_stat) < 0) {
	 fprintf(stderr, "%s: Cannot stat directory %s.\n", PROGNAME, pathname);
         ERRX;
      }
      
      /* check if the dir_entry is "." or ".." to avoid loops */
      founddotdir = !(strncmp(dir_entry->d_name, ".", MAXPATHLEN)) || \
		    !(strncmp(dir_entry->d_name, "..", MAXPATHLEN));
      
      /* if it's a directory - search it too */ 
      if (S_ISDIR(dir_stat.st_mode) && !founddotdir) {
         if (getcwd(buf, MAXPATHLEN) == NULL) {
	    fprintf(stderr, "%s: Cannot get current working directory.\n", PROGNAME);
	    WARNX;
	 }
         
         scan_directory(dir_entry->d_name, pattern, options);
         
         /* return to the last scanning dir */
         if (chdir(buf) == -1) {
	    fprintf(stderr, "%s: Cannot return to previous scanning directory.\n", PROGNAME);
	    WARNX;
	 }
      }
          
      /* skip the entry if it's a directory, or if it is the "." or ".." directories */
      if (founddotdir || S_ISDIR(dir_stat.st_mode))
	 continue;
   
      /* save the entry if it matches the given pattern */
      if (options & OPTION_REGEX) {
	 if ((!regexec(&myregex, dir_entry->d_name, 0, NULL, 0)) && \
	     (strcasestr(dir_entry->d_name, PKG_EXTENSION) != NULL))
	    found_pkg = 1;
      } else
	 if ((strcasestr(dir_entry->d_name, pattern) != NULL)	    && \
	     (strcasestr(dir_entry->d_name, PKG_EXTENSION) != NULL))
	    found_pkg = 1;

      if (found_pkg) {
	 count++;

         if (getcwd(buf, MAXPATHLEN) == NULL) {
	    fprintf(stderr, "%s: Cannot get current working directory.\n", PROGNAME);
	    WARNX;
	 }

	 entry->pkg_fields[PKG_NAME] = strndup(dir_entry->d_name, MAXPATHLEN);
	 entry->pkg_fields[PKG_CAT]  = strndup(buf, MAXPATHLEN);

	 if ((entry->pkg_fields[PKG_NAME] == NULL) || (entry->pkg_fields[PKG_CAT] == NULL)) {
	    fprintf(stderr, "%s: Cannot allocate memory.\n", PROGNAME);
	    ERRX;
	 }

         pkg_insert(entry, 0);

	 free(entry->pkg_fields[PKG_NAME]);
	 free(entry->pkg_fields[PKG_CAT]);
      }
   }
   
   free(entry);
   (void)closedir(dir);
   
   if (options & OPTION_REGEX)
      regfree(&myregex);
     
   return(count);
}

==> pkg_add_it-1.3.2.tar.gz: pkg_add_it-1.3.2/scan_dir.c <==
/*-
 * Copyright (c) 2010  Marin Atanasov Nikolov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>

#include <stdio.h>
#include <regex.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>

#include "main.h"
#include "pkg.h"
#include "scan_dir.h"

/* dir_exists
 *
 * checks for existence of a directory 
 */
int
dir_exists(const char *dirname)
{
   struct stat dir_stat;
   
   stat (dirname, &dir_stat);
   
   return S_ISDIR(dir_stat.st_mode);
}

/* scan_directory
 *
 * Recursively scans the specified directory for the package pattern.
 *
 * During local package search the pkg_fields[PKG_CAT] element is
 * used to store the full path to the package location.
 *
 * Returns the number of found packages.
 */
int
scan_directory(const char *pathname, const char *pattern, int options)
{
   PKG	  *entry;
   regex_t myregex;
   DIR	  *dir;
   char	   buf[MAXPATHLEN + 1]; 
   struct  dirent *dir_entry;
   struct  stat dir_stat;
   int     founddotdir, found_pkg, i;
   static unsigned count = 0;
   
   entry = (PKG *) malloc(sizeof(PKG));

   if (entry == NULL) {
      fprintf(stderr, "%s: Cannot allocate memory.\n", PROGNAME);
      ERRX;
   }

   entry->flags = 0;
   entry->pkg_num = 0;
   STAILQ_INIT(&entry->deps_head);

   for (i = 0; i < PKG_FIELDS; i++)
      entry->pkg_fields[i] = NULL;

   if (options & OPTION_REGEX)
      if (regcomp(&myregex, pattern, REG_ICASE | REG_NOSUB | \
	         ((options & OPTION_REGEX_EXTENDED) ? REG_EXTENDED : REG_BASIC)) != 0) {
	 fprintf(stderr, "%s: Cannot compile the given regex.\n", PROGNAME);
	 ERRX;
      }

   if (!(dir = opendir(pathname))) {
      WARNX;
      fprintf(stderr, "%s: Skipping directory %s\n", PROGNAME, pathname);
      return(0);
   }

   if (chdir(pathname) == -1) {
      WARNX;
      fprintf(stderr, "%s: Skipping directory %s\n", PROGNAME, pathname);
      return(0);
   }
   
   while ((dir_entry = readdir(dir))) {
      found_pkg = 0;

      if (stat(dir_entry->d_name, &dir_stat) < 0) {
	 fprintf(stderr, "%s: Cannot stat directory %s.\n", PROGNAME, pathname);
         ERRX;
      }
      
      /* check if the dir_entry is "." or ".." to avoid loops */
      founddotdir = !(strncmp(dir_entry->d_name, ".", MAXPATHLEN)) || \
		    !(strncmp(dir_entry->d_name, "..", MAXPATHLEN));
      
      /* if it's a directory - search it too */ 
      if (S_ISDIR(dir_stat.st_mode) && !founddotdir) {
         if (getcwd(buf, MAXPATHLEN) == NULL) {
	    fprintf(stderr, "%s: Cannot get current working directory.\n", PROGNAME);
	    WARNX;
	 }
         
         scan_directory(dir_entry->d_name, pattern, options);
         
         /* return to the last scanning dir */
         if (chdir(buf) == -1) {
	    fprintf(stderr, "%s: Cannot return to previous scanning directory.\n", PROGNAME);
	    WARNX;
	 }
      }
          
      /* skip the entry if it's a directory, or if it is the "." or ".." directories */
      if (founddotdir || S_ISDIR(dir_stat.st_mode))
	 continue;
   
      /* save the entry if it matches the given pattern */
      if (options & OPTION_REGEX) {
	 if ((!regexec(&myregex, dir_entry->d_name, 0, NULL, 0)) && \
	     (strcasestr(dir_entry->d_name, PKG_EXTENSION) != NULL))
	    found_pkg = 1;
      } else
	 if ((strcasestr(dir_entry->d_name, pattern) != NULL)	    && \
	     (strcasestr(dir_entry->d_name, PKG_EXTENSION) != NULL))
	    found_pkg = 1;

      if (found_pkg) {
	 count++;

         if (getcwd(buf, MAXPATHLEN) == NULL) {
	    fprintf(stderr, "%s: Cannot get current working directory.\n", PROGNAME);
	    WARNX;
	 }

	 entry->pkg_fields[PKG_NAME] = STRNDUP(dir_entry->d_name, MAXPATHLEN);
	 entry->pkg_fields[PKG_CAT]  = STRNDUP(buf, MAXPATHLEN);

	 if ((entry->pkg_fields[PKG_NAME] == NULL) || (entry->pkg_fields[PKG_CAT] == NULL)) {
	    fprintf(stderr, "%s: Cannot allocate memory.\n", PROGNAME);
	    ERRX;
	 }

         pkg_insert(entry, 0);

	 free(entry->pkg_fields[PKG_NAME]);
	 free(entry->pkg_fields[PKG_CAT]);
      }
   }
   
   free(entry);
   (void)closedir(dir);
   
   if (options & OPTION_REGEX)
      regfree(&myregex);
     
   return(count);
}
